@@ -15,6 +15,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <bonobo/bonobo-i18n.h>
+#include <bonobo/bonobo-dock-item.h>
 #include <bonobo/bonobo-ui-xml.h>
 #include <bonobo/bonobo-ui-util.h>
 #include <bonobo/bonobo-ui-engine.h>
@@ -282,6 +283,32 @@ label_same (GtkBin *menu_widget, const char *txt)
 		!strcmp (((GtkLabel *)label)->label, txt);
 }
 
+static gboolean
+widget_has_accel (GtkWidget       *widget,
+		  GtkAccelGroup   *accel_group,
+		  guint           key, 
+		  GdkModifierType mods)
+{
+	int i;
+	guint n_entries;
+	GList *l_tmp, *l_closures = gtk_widget_list_accel_closures (widget);
+	GtkAccelGroupEntry *entry = gtk_accel_group_query (accel_group,
+							   key, mods,
+							   &n_entries);
+
+	if (n_entries)
+		for (l_tmp = l_closures; l_tmp; l_tmp = l_tmp->next)
+	  		for (i=0; i<n_entries; i++) 
+			  	if (entry[i].closure == l_tmp->data) {
+					g_list_free (l_closures);
+					return TRUE;
+				}
+	
+	g_list_free (l_closures);
+
+	return FALSE;
+}
+
 static void
 impl_bonobo_ui_sync_menu_state (BonoboUISync *sync,
 				BonoboUINode *node,
@@ -392,11 +419,12 @@ impl_bonobo_ui_sync_menu_state (BonoboUISync *sync,
 		if (!key) /* FIXME: this looks strange */
 			return;
 
-		gtk_widget_add_accelerator (menu_widget,
-					    "activate",
-					    sync_menu->accel_group,
-					    key, mods,
-					    GTK_ACCEL_VISIBLE);
+	  	if (! widget_has_accel (menu_widget, sync_menu->accel_group, key, mods))
+			gtk_widget_add_accelerator (menu_widget,
+						    "activate",
+						    sync_menu->accel_group,
+						    key, mods,
+						    GTK_ACCEL_VISIBLE);
 	}
 
 	bonobo_ui_engine_queue_update (
@@ -506,8 +534,7 @@ impl_bonobo_ui_sync_menu_build (BonoboUISync     *sync,
 
 	if (bonobo_ui_node_has_name (node, "separator")) {
 
-		menu_widget = gtk_menu_item_new ();
-		gtk_widget_set_sensitive (menu_widget, FALSE);
+		menu_widget = gtk_separator_menu_item_new ();
 
 	} else if (bonobo_ui_node_has_name (node, "control")) {
 
@@ -672,8 +699,7 @@ impl_bonobo_ui_sync_menu_build_placeholder (BonoboUISync     *sync,
 	if (!parent) /* A popup without a GtkMenu inserted as yet. */
 		return NULL;
 
-	widget = gtk_menu_item_new ();
-	gtk_widget_set_sensitive (widget, FALSE);
+	widget = gtk_separator_menu_item_new ();
 
 	gtk_menu_shell_insert (GTK_MENU_SHELL (parent),
 			       GTK_WIDGET (widget), (*pos)++);
@@ -722,6 +748,7 @@ impl_dispose (GObject *object)
 	BonoboUISyncMenu *sync = (BonoboUISyncMenu *) object;
 
 	if (sync->menu) {
+		gtk_widget_destroy (GTK_WIDGET (sync->menu));
 		g_object_unref (sync->menu);
 		sync->menu = NULL;
 	}
@@ -758,12 +785,21 @@ impl_bonobo_ui_sync_menu_update_root (BonoboUISync *sync,
 				      BonoboUINode *root)
 {
 	BonoboUISyncMenu *smenu = BONOBO_UI_SYNC_MENU (sync);
+	const char *txt;
+	gboolean    detachable;
 
 	if (bonobo_ui_node_has_name (root, "menu") &&
 	    smenu->menu_dock_item)
 		bonobo_ui_sync_do_show_hide (
 			sync, root, NULL,
 			smenu->menu_dock_item);
+
+	if ((txt = bonobo_ui_node_peek_attr (root, "behavior")) &&
+	    strstr (txt, "detachable"))
+		detachable = TRUE;
+	else
+	    detachable = bonobo_ui_preferences_get_menubar_detachable ();
+	bonobo_dock_item_set_locked (BONOBO_DOCK_ITEM (smenu->menu_dock_item), !detachable);
 }
 
 static void
@@ -832,7 +868,7 @@ impl_bonobo_ui_sync_menu_get_attached (BonoboUISync *sync,
 	return get_item_widget (widget);
 }
 
-GtkWidget *
+static GtkWidget *
 impl_bonobo_ui_sync_menu_wrap_widget (BonoboUISync *sync,
 				      GtkWidget    *custom_widget)
 {
